/*
 * Copyright (C) 2003  Diez B. Roggisch [deets@web.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: HashMapModel.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.dbroggisch.display;


import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.barracudamvc.core.comp.AbstractTemplateModel;
import org.barracudamvc.core.comp.TemplateDirective;

public class HashMapModel extends AbstractTemplateModel implements LightweightTemplateModel {

    protected Map values;
    protected String modelName;

    public String getName()
    {
        return modelName;
    }

    //csc_030703.1 - added
    public Object getItem(TemplateDirective td) {
        return getItem(td.getKeyName());
    }


    public Object getItem(String key) {
        if(values != null && values.containsKey(key)) {
            return values.get(key);
        }
        return super.getItem(key);
    }

    public void setName(String name)
    {
        modelName = name;
    }

    public HashMapModel() {
        this(null, null);
    }

    public HashMapModel(String name, Map mp) {
        modelName = name;
        if(mp != null) {
            values = mp;
        } else {
            values = new HashMap();
        }
    }

    public HashMapModel(String name) {
        this(name, null);
    }


    public HashMapModel(Map mp) {
        this(null, mp);
    }



    public void setItem(String key, Object value) {
        values.put(key, value);
    }


    public Map getItems() {
        return values;
    }

    public void setItems(Map v) {
        values = v;
    }

    public void clear() {
        values.clear();
    }

    public Set keySet() {
        return values.keySet();
    }
}
