/*
 * Copyright (C) 2003  Diez B. Roggisch [deets@web.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: DateFilter.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.dbroggisch.display.filters;

import java.lang.reflect.Field;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Locale;

import org.apache.log4j.Logger;

public class DateFilter implements Filter {
    private static Logger logger = Logger.getLogger(DateFilter.class.getName());

        private String formatString;
        private DateFormat dateFormat;

        private Locale loc;


        public DateFilter(String formatString, Locale loc) {
                if(loc == null) {
                        loc = Locale.getDefault();
                } else {
                        this.loc = loc;
                }
                this.formatString = formatString;
        }

        public DateFilter(String formatString) {
                this(formatString, null);
        }

        public DateFilter() {
                this(null);
        }


        public Object filter(Object obj, FilterContext context)
                throws FilterException
        {
                if(obj == null) {
                        return null;
                }
                if(obj instanceof java.util.Date) {
                        if(dateFormat == null) {
                                dateFormat = new SimpleDateFormat(formatString, loc);
                        }
                        return dateFormat.format((java.util.Date)obj);
                } else {
                        throw new FilterException("Not a java.util.Date");
                }
        }


        public Filter configure(Object obj)
                throws FilterException
        {
                org.barracudamvc.contrib.dbroggisch.display.filters.dtd.Date xmlObj = (org.barracudamvc.contrib.dbroggisch.display.filters.dtd.Date)obj;
                this.formatString = xmlObj.getValue();
                String locale = xmlObj.getLocale();
                if(locale != null && !locale.trim().equals("")) {
                        try {
                                Field f = java.util.Locale.class.getDeclaredField(locale);
                                Locale loc = (Locale)f.get(null);
                                if(logger.isDebugEnabled()) {
                                        logger.debug("Setting locale " + locale);
                                }
                                this.loc = loc;
                        }
                        catch(NoSuchFieldException ex) {
                                logger.error(ex.getMessage());
                                ex.printStackTrace();
                                throw new FilterException("Error configuring DateFilter");
                        }
                        catch(IllegalAccessException ex) {
                                logger.error(ex.getMessage());
                                ex.printStackTrace();
                                throw new FilterException("Error configuring DateFilter");
                        }
                }
                return this;

        }

}
