/*
 * Copyright (C) 2003  Diez B. Roggisch [deets@web.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: LookupFilter.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.dbroggisch.display.filters;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.barracudamvc.contrib.dbroggisch.display.filters.dtd.Entry;
import org.barracudamvc.contrib.dbroggisch.display.filters.dtd.Lookup;

/* This class uses a given Map to lookup the
 * objects meaning.
 *
 * @author <a href="mailto:deets@web.de">Diez B. Roggisch</a>
 * @version 1.0
 */
public class LookupFilter implements Filter {
    protected static final Logger logger = Logger.getLogger(LookupFilter.class.getName());

    private Map lookupMap;

    public LookupFilter() {
        this(null);
    }

    public LookupFilter(Map lookupMap) {
        this.lookupMap = lookupMap;
    }

    /** Looks up the given key. If the key doesn't exist, <code>null</code>
     * returned. If the lookupMap is <code>null</code>, the key itself will be returned.
     *
     * @param key The key to lookup.
     * @return the mapped objet.
     */
    public Object filter(Object key, FilterContext context)
        throws FilterException
    {
        if(key != null && lookupMap != null) {
            return lookupMap.get(key);
        }
        return key;
    }

    public Filter configure(Object obj)
        throws FilterException
    {
        lookupMap = new HashMap();
        try {
            Lookup xmlObj = (Lookup)obj;
            List entries = xmlObj.getEntryList();
            for(Iterator it = entries.iterator(); it.hasNext();) {
                Entry entry = (Entry)it.next();
                Object key;
                if(entry.getType().equals("Integer")) {
                    key = new Integer(Integer.parseInt(entry.getKey()));
                } else if(entry.getType().equals("Boolean")) {
                    key = Boolean.valueOf(entry.getKey());
                } else if(entry.getType().equals("Class")) {
                    try {
                        key = Class.forName(entry.getKey());
                    } catch(ClassNotFoundException ex) {
                        StringWriter sw = new StringWriter();
                        ex.printStackTrace(new PrintWriter(sw));
                        logger.error(sw.toString());
                        throw new FilterException(ex.getMessage());
                    }
                } else {
                    key = entry.getKey();
                }
                lookupMap.put(key, entry.getValue());
            }
        } catch(ClassCastException ex) {
            ex.printStackTrace();
        }
        return this;
    }
}
