/*
 * Copyright (C) 2003  Diez B. Roggisch [deets@web.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: Page.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.dbroggisch.page;

import java.io.IOException;

import javax.servlet.ServletException;

import org.barracudamvc.core.event.ControlEventContext;
import org.barracudamvc.core.event.EventException;
import org.barracudamvc.core.event.ViewEventContext;


/**
 * The Page interface is the central interface for the Page-pattern.
 * As you might have guessed by the name.
 * Basically the page-pattern is a way to get rid of render-events and provides a convenient way to
 * encapsulate and inherit code for rendering a html page.
 *
 * @author <a href="mailto:deets@web.de">Diez B. Roggisch</a>
 * @version 1.0
 */
public interface Page {

    /**
     * The name of this method is somewhat misleading -
     *
     * @param ctx a <code>ControlEventContext</code> value
     */
    public void createDefaultModels(ControlEventContext ctx);

    /**
     * Describe <code>render</code> method here.
     *
     * @param ctx a <code>ViewEventContext</code> value
     * @exception EventException if an error occurs
     * @exception javax.servlet.ServletException if an error occurs
     * @exception java.io.IOException if an error occurs
     */
    public void render(ViewEventContext ctx)
            throws EventException, ServletException, IOException;

    /**
     * Describe <code>handOver</code> method here.
     *
     * @param ctx a <code>ControlEventContext</code> value
     * @param p a <code>Page</code> value
     */
    public void handOver(ControlEventContext ctx, Page p);

}
