/*
 * Copyright (C) 2003  Diez B. Roggisch [deets@web.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: GroupFormElement.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.dbroggisch.repopulation;

import org.barracudamvc.core.comp.AbstractTemplateModel;
import org.barracudamvc.core.comp.BInput;
import org.barracudamvc.core.comp.BToggleButton;
import org.barracudamvc.core.comp.ItemMap;
import org.barracudamvc.core.comp.IterativeModel;
import org.barracudamvc.core.comp.ListModel;
import org.barracudamvc.core.comp.ListSelectionModel;
import org.barracudamvc.core.forms.FormType;
import org.barracudamvc.core.forms.FormValidator;

public class GroupFormElement extends SelectFormElement implements ChildModelElement
{
    public String KEY = "KEY";
    public String VALUE = "VALUE";

    public IterativeModel getChildModel(String parentName, String prefix) {
        return new ChildModel(parentName, getName(), prefix);
    }

    public GroupFormElement(ListModel lm,
                 java.lang.String ikey,
                 FormType itype,
                 java.lang.Object idefaultVal,
                 FormValidator ivalidator,
                 boolean iallowMultiples) {
        super(lm, ikey, itype, idefaultVal, ivalidator, iallowMultiples);
    }


    public class ChildModel extends AbstractTemplateModel implements IterativeModel {

        String _name, _parent_name, _child_name, _prefix;
        ListSelectionModel _lsm;
        ListModel _lm;
        int _count;
        Object _key, _value;
        boolean _allows_multiple;

        public ChildModel(String parentName, String childName, String prefix) {
            _parent_name = parentName;
            _child_name = childName;
            _name = _parent_name + "_" + _child_name;
            if(prefix != null) {
                _prefix = prefix;
            } else {
                _prefix = "";
            }
            _allows_multiple = allowMultiples();
        }

        public String getName() {
            return _name;
        }


        public Object getItem(String key) {
            if(key.equals(KEY)) {
                return _key;
            } else if(key.equals(VALUE)) {
                return _value;
            } else {
                return super.getItem(key);
            }
        }

        public void preIterate() {
            _lsm = getListSelectionModel();
            _lm = getListModel();
            _count = -1;
        }

        public void postIterate() {
        }

        public boolean hasNext() {
            return _lm.getSize() > 0 && _count < _lm.getSize() - 1;
        }

        public void loadNext() {
            _count++;
            Object item = _lm.getItemAt(_count);
            BToggleButton btog = new BToggleButton();
            btog.setName(_child_name);
            btog.setSelected(_lsm.isSelectedIndex(_count));
            if(item instanceof ItemMap) {
                ItemMap im = (ItemMap)item;
                btog.setValue("" + im.getKey());
                _value = im.getValue();
            } else {
                btog.setValue("" + item);
                _value = item;
            }
            if(_allows_multiple) {
                btog.setType(BInput.CHECKBOX);
            } else {
                btog.setType(BInput.RADIO);
            }
            _key = btog;
        }
    }

}
