/*
 * Copyright (C) 2003  Diez B. Roggisch [deets@web.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: RepopulationPage.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.dbroggisch.examples.view.pages;

import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import javax.servlet.ServletException;
import org.w3c.dom.Document;
//import org.enhydra.xml.xmlc.XMLCStdFactory;
import org.barracudamvc.core.comp.BAction;
import org.barracudamvc.core.comp.BComponent;
import org.barracudamvc.core.comp.BTemplate;
import org.barracudamvc.core.comp.DefaultTemplateView;
import org.barracudamvc.core.comp.TemplateModel;
import org.barracudamvc.core.comp.ViewContext;
import org.barracudamvc.core.event.EventException;
import org.barracudamvc.core.util.dom.DefaultDOMLoader;
import org.barracudamvc.contrib.dbroggisch.display.HashMapModel;
import org.barracudamvc.contrib.dbroggisch.repopulation.RepopulationFormMap;
import org.apache.log4j.Logger;

import org.barracudamvc.contrib.dbroggisch.examples.controller.events.*;
import org.barracudamvc.contrib.dbroggisch.examples.model.forms.*;
import org.barracudamvc.contrib.dbroggisch.examples.view.xmlc.*;


public class RepopulationPage extends ExampleBasePage {

    private static final Logger logger = Logger.getLogger(RepopulationPage.class.getName());

    RepopulationFormMap _rfm;

    public void setRFM(RepopulationFormMap rfm) {
        _rfm = rfm;
    }

    public void render(BComponent root, ViewContext vec, Document pDoc)
            throws EventException, ServletException, IOException {
        super.render(root, vec, pDoc);
        RepopulationHTML doc = (RepopulationHTML) pDoc;

        // Fetch the map that's supposed to act as template model.
        ExampleFormMap cfm = (ExampleFormMap)_rfm;
        BTemplate loginComp = new BTemplate();

        loginComp.setView(new DefaultTemplateView(doc.getElementTestHook()));

        // Add the map as model to the BTemplate
        loginComp.addModel(cfm);

        // It is possible that the map contains multiple choice elements represented by their respective model.
        // That happens when you use CheckBoxes or Radiobuttons instead of combo-boxes.
        // For now, Barracuda lacks the possibility to add these automatically, so we have to do it.
        List cModels = cfm.getChildModels();
        for(Iterator it = cModels.iterator(); it.hasNext();) {
            loginComp.addModel((TemplateModel)it.next());
        }

        // A simple hashmap based model for the buttons.
        HashMapModel hmm = new HashMapModel();
        hmm.setName("ControlModel");
        hmm.setItem("SUBMIT", new BAction(new Repopulation()));
        loginComp.addModel(hmm);
        root.addChild(loginComp);
    }

    public Document getDocument(Locale iLocale) {
        //return new XMLCStdFactory(null, null).create(RepopulationHTML.class);

        Document doc = null;
        //load the localized DOM template
        try {doc = DefaultDOMLoader.getGlobalInstance().getDOM(RepopulationHTML.class, iLocale);}
        catch (IOException ioe) {logger.fatal ("Fatal Error loading DOM template:", ioe);}
        return doc;
    }

}
