/*
 * Copyright (C) 2003  Stefan Armbruster [stefan@armbruster-it.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: DataObjectGateway.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.sam.data;

import org.apache.log4j.Logger;
import org.barracudamvc.core.event.DefaultEventGateway;
import org.barracudamvc.core.event.helper.EventConnectorFactory;

/**
 * Event handlers (both Controller and View) for the
 * Main screen
 */
public abstract class DataObjectGateway extends DefaultEventGateway {

    //public constants
    protected static Logger logger = Logger.getLogger(DataObjectGateway.class.getName());

    protected abstract Class getDefaultHandler();
    protected abstract Class getDefaultEvent();
    protected abstract Class getSaveHandler();
    protected abstract Class getSaveEvent();
    protected abstract Class getDeleteHandler();
    protected abstract Class getDeleteEvent();
    protected abstract Class getNewHandler();
    protected abstract Class getNewEvent();
    protected abstract Class getListHandler();
    protected abstract Class getListEvent();
    protected abstract Class getRenderEvent();
    protected abstract Class getRenderHandler();



    public DataObjectGateway() {
        logger.debug("registering Handlers:");
        logger.debug("DefaultEvent: " + getDefaultEvent().getName());
        specifyLocalEventInterests(new EventConnectorFactory(getDefaultHandler()), getDefaultEvent());
        specifyLocalEventInterests(new EventConnectorFactory(getSaveHandler()), getSaveEvent());
        specifyLocalEventInterests(new EventConnectorFactory(getDeleteHandler()), getDeleteEvent());
        specifyLocalEventInterests(new EventConnectorFactory(getNewHandler()), getNewEvent());
        if (getListEvent() != null) {
                specifyLocalEventInterests(new EventConnectorFactory(getListHandler()), getListEvent());
            }
        specifyLocalEventInterests(new EventConnectorFactory(getRenderHandler()), getRenderEvent());
        //specifyLocalEventInterests(new EventConnectorFactory(getHandler()), Event);
    }
}
