/*
 * Copyright (C) 2003  Stefan Armbruster [stefan@armbruster-it.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: IteratorModel.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.sam.models;

import java.util.Iterator;

import org.apache.log4j.Logger;
import org.barracudamvc.core.comp.AbstractIterativeTemplateModel;

/** iterates over a Map, uses the special keys "Key" and "Value" for returning key-value-pairs
 * @author Stefan Armbruster
 * @version $Id: IteratorModel.java 114 2005-12-09 15:51:51Z christianc $
 */
public abstract class IteratorModel extends AbstractIterativeTemplateModel {

    protected static Logger logger = Logger.getLogger(IteratorModel.class.getName());

    /** data to iterate  */
    /** name of the model */
    protected String name;
    protected Iterator _iter;

        /** current element during iteration, might be casted to appropiate class */
    protected Object _current;

    /** initialize the model
     * @param name Name of the model
     * @param iter Iterator to iterate
     */
    public IteratorModel(String name, Iterator iter) {
        _iter = iter;
        setName(name);
    }

    public IteratorModel(String name ) {
        _iter = null;
        setName(name);
    }

    /** get the name of the model
     * @return name of the model
     */
    public String getName() {
        return name;
    }

     public void setIterator(Iterator iter) {
        _iter = iter;
     }

    /** set the name of the model
     * @param name name of the model
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     *
     */
    public boolean hasNext() {
        return _iter.hasNext();
    }

    /** empty implementation, does nothing. Only reason for being here is to fully
     * implement IterativeModel.
     */
    public void loadNext() {
        _current = _iter.next();
    }

}
