/*
 * Copyright (C) 2003  Stefan Armbruster [stefan@armbruster-it.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: SubModelGateway.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.sam.examples.submodel;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.servlet.ServletException;

import org.apache.log4j.Logger;
import org.barracudamvc.contrib.dbroggisch.display.IterativeCollectionModel;
import org.barracudamvc.contrib.sam.examples.events.RenderSubModel;
import org.barracudamvc.contrib.sam.examples.events.SubModel;
import org.barracudamvc.contrib.sam.examples.xmlc.submodelHTML;
import org.barracudamvc.contrib.sam.models.IterativeSubModel;
import org.barracudamvc.contrib.sam.xmlform.ModifyDomViewHandler;
import org.barracudamvc.core.comp.TemplateModel;
import org.barracudamvc.core.comp.ViewContext;
import org.barracudamvc.core.event.DefaultEventGateway;
import org.barracudamvc.core.event.EventException;
import org.barracudamvc.core.event.helper.EventConnectorFactory;
import org.barracudamvc.core.event.helper.EventForwardingFactory;


/**
 * @author stefan
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class SubModelGateway extends DefaultEventGateway {
    
    protected static Logger logger = Logger.getLogger(SubModelGateway.class);
    public SubModelGateway() {
        super();
        logger.info("registering SubModelGateway's listeners");
        specifyLocalEventInterests(new EventForwardingFactory(new RenderSubModel()),  SubModel.class);
        specifyLocalEventInterests(new EventConnectorFactory(RenderSubModelHandler.class), RenderSubModel.class);
    }
    
    public static class RenderSubModelHandler extends ModifyDomViewHandler {
        
        protected static Logger logger = Logger.getLogger(RenderSubModelHandler.class);

        public Object getTemplateModels(ViewContext vc) throws EventException, ServletException, IOException {
            
            
            Collection data = new ArrayList();
            Collection subCollection1 = new ArrayList();
            subCollection1.add("Item Sub 1-1");
            subCollection1.add("Item Sub 1-2");
            data.add(subCollection1);
            Collection subCollection2 = new ArrayList();
            subCollection2.add("Item Sub 2-1");
            subCollection2.add("Item Sub 2-2");
            data.add(subCollection2);
            
            List listOfModels = new ArrayList();
            
            TemplateModel parentModel = new IterativeCollectionModel("parent", data) {
                
                public void preIterate() {
                    RenderSubModelHandler.logger.debug("preIterate for model " + getName() + " called!");
                    super.preIterate();
                }
                
                public void postIterate() {
                    RenderSubModelHandler.logger.debug("postIterate for model " + getName() + " called!");
                    super.postIterate();
                }
                
                public Object getItem(String key) {
                    return _current;
                }
            };
            listOfModels.add(parentModel);
            
            IterativeSubModel childModel = new IterativeSubModel("child", null, parentModel ) {
                
                public void preIterate() {
                    RenderSubModelHandler.logger.debug("preIterate for model " + getName() + " called!");
                    super.preIterate();
                }

                public void postIterate() {
                    RenderSubModelHandler.logger.debug("postIterate for model " + getName() + " called!");
                    super.postIterate();
                }

            };
            listOfModels.add(childModel);
            
//            listOfModels.add(new IterativeCollectionModel("simple", subCollection2) {
//                
//                public void preIterate() {
//                    super.preIterate();
//                    logger.error("preIterate called!");
//                }
//                
//                
//                public void postIterate() {
//                    super.postIterate();
//                    logger.error("postIterate called!");
//                }
//
//                public Object getItem(String key) {
//                    return _current;
//                }
//            });
            
            return listOfModels;
          
        }

        /* (non-Javadoc)
         * @see org.barracudamvc.contrib.sam.xmlform.ModifyDomViewHandler#getTemplateClass(org.barracudamvc.core.comp.ViewContext)
         */
        public Class getTemplateClass(ViewContext vc) throws EventException, ServletException, IOException {
            return submodelHTML.class;
        }
        
    }


}
