/*
 * Copyright (C) 2003 Stefan Armbruster [stefan@armbruster-it.de]
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General
 * Public License as published by the Free Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this library; if not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * $Id: XmlFormFactory.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.sam.xmlform;

import java.io.InputStream;
import java.util.Hashtable;
import java.util.Map;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;

import org.apache.log4j.Logger;
import org.barracudamvc.contrib.sam.xmlform.dtd.Form;
import org.barracudamvc.contrib.sam.xmlform.dtd.FormImpl;
import org.barracudamvc.core.event.ControlEventContext;

/**
 * XmlFormFactory holds a map of parsed forms. This way, the number of xml parsings is reduced as much as possible
 */
public class XmlFormFactory {

    protected static Logger logger = Logger.getLogger(XmlFormFactory.class.getName());

    /** Map holding all already parsed XMLForms */
    protected static Map parsedXmlFiles = new Hashtable();

    /** Hidden constructor to prevent any instantiazion */
    protected XmlFormFactory() {
    }

    /**
     * If the requested Form is already stored in {@link #parsedXmlFiles}it is simply returned. Otherwise the XML file
     * is stored in parsed form. Parsing is done by Zeus generated classes.
     */
    public static Form get(ControlEventContext context, String filename) {
        try {
            Form form = (Form) parsedXmlFiles.get(filename);
            if (form == null) {
                if (logger.isDebugEnabled()) logger.debug(filename + " is currently not in map, trying to load");
                ServletConfig config = context.getConfig();
                ServletContext sc = config.getServletContext();
                if (logger.isDebugEnabled()) logger.debug("URL is " + sc.getResource(filename));
                InputStream is = sc.getResourceAsStream(filename);
                form = FormImpl.unmarshal(is);
                parsedXmlFiles.put(filename, form);
            }
            return form;
        } catch (Exception ex) {
            String msg = filename + ": " + ex.getMessage();
            logger.error("Processing " + msg, ex);
            throw new RuntimeException(msg, ex);
        }
    }

}