/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: DeferredValidationException.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.core.forms;

import java.util.List;

/**
 * This class defines a deferred validation exception. It's just 
 * like a standard validation exception, except that validation
 * continues (so that we can collect all possible validation
 * errors on a form at one time).
 */
public class DeferredValidationException extends ValidationException {

    protected List subExceptions = null;

    /**
     * The noargs public contructor for DeferredValidationException
     */
    public DeferredValidationException () {super(null);}

    /**
     * The public contructor for DeferredValidationException
     *
     * @param source the object which caused this error (usually a
     *        FormElement)
     */
    public DeferredValidationException (Object source) {super(source, null);}

    /**
     * The public contructor for DeferredValidationException
     *
     * @param source the object which caused this error (usually a
     *        FormElement)
     * @param s a String describing the exception
     */
    public DeferredValidationException (Object source, String s) {super(source, s, null);}

    /**
     * The public contructor for DeferredValidationException
     *
     * @param source the object which caused this error (usually a
     *        FormElement)
     * @param s a String describing the exception
     * @param ibaseException the original exception to wrap within this exception
     */
    public DeferredValidationException (Object source, String s, Exception ibaseException) {super(source, s, ibaseException);}

}
