/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: FormMap.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.core.forms;

import java.util.*;

import javax.servlet.ServletRequest;

import org.barracudamvc.plankton.data.StateMap;

/**
 * <p>A FormMap is used to provide a virtual representation of a
 * form. It can contain any number of unique FormElements, and
 * it can also be associated with FormValidators. The primary
 * function of a form map is to:
 *
 * <ul>
 *         <li>define the map (with its elements and validators)</li>
 *         <li>actually populate the map (from either a ServletRequest
 *            or a StateMap)</li>
 *         <li>validate the map (by invoking all the validators associated
 *            with the form and all its elements)</li>
 *        <li>provide convenience methods to access the underlying values
 *            of the form elements contained in this map</li>
 * </ul>
 *
 * @author  Christian Cryder [christianc@granitepeaks.com]
 * @author  Diez B. Roggisch <diez.roggisch@artnology.com>
 * @author  Jacob Kjome <hoju@visi.com>
 * @version %I%, %G%
 * @since   1.0
 */
public interface FormMap extends StateMap {

    public FormElement defineElement(FormElement element);
    public FormElement defineElement(String key, FormElement element);
    public FormValidator defineValidator(FormValidator validator);
    public FormMap map(ServletRequest req);
//    public FormMap map(ServletRequest req, String prefix);
//    public FormMap map(ServletRequest req, Locale loc);
//    public FormMap map(ServletRequest req, String prefix, Locale loc);
    public FormMap map(StateMap map);
//    public FormMap map(StateMap map, String prefix);
//    public FormMap map(StateMap map, Locale loc);
//    public FormMap map(StateMap map, String prefix, Locale loc);
    public FormElement mapElement(String key, Object origVal);
//    public FormElement mapElement(String key, Object origVal, Locale loc);
    public FormMap validate(boolean deferExceptions) throws ValidationException;
    public FormMap validateElements(boolean deferExceptions) throws ValidationException;
    public FormMap validateForm(boolean deferExceptions) throws ValidationException;
    public boolean exists(String key);
    public FormElement getElement(String key);
    public Map getElements();
//    public Map getMappedElements();
//    public Map getElementVals();
    public void setVal(String key, Object val);
    public Object getVal(String key);
    public Object getVal(String key, Object dflt);
    public Object[] getVals(String key);
    public Object[] getVals(String key, Object[] dflt);
    public void setFormMapper(FormMapper mapper);
    public FormMapper getFormMapper();
    public void setLocale(Locale loc);
    public Locale getLocale();
//    public String getStringVal(String key);
//    public String getStringVal(String key, String dflt);
//    public Boolean getBooleanVal(String key);
//    public Boolean getBooleanVal(String key, Boolean dflt);
//    public Integer getIntegerVal(String key);
//    public Integer getIntegerVal(String key, Integer dflt);
//    public Date getDateVal(String key);
//    public Date getDateVal(String key, Date dflt);
//    public Long getLongVal(String key);
//    public Long getLongVal(String key, Long dflt);
//    public Short getShortVal(String key);
//    public Short getShortVal(String key, Short dflt);
//    public Double getDoubleVal(String key);
//    public Double getDoubleVal(String key, Double dflt);
//    public Float getFloatVal(String key);
//    public Float getFloatVal(String key, Float dflt);
//    public boolean isNull(Object val);


    /**
     * @clientCardinality 0..*
     */
    /*#FormElement lnkFormElement;*/

}
