/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: TestScreen.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.examples.ex3;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.barracudamvc.core.event.BaseEvent;
import org.barracudamvc.core.event.BaseEventListener;
import org.barracudamvc.core.event.ControlEventContext;
import org.barracudamvc.core.event.DefaultBaseEventListener;
import org.barracudamvc.core.event.DefaultEventGateway;
import org.barracudamvc.core.event.DefaultListenerFactory;
import org.barracudamvc.core.event.DispatchQueue;
import org.barracudamvc.core.event.EventException;
import org.barracudamvc.core.event.ListenerFactory;
import org.barracudamvc.core.event.ViewEventContext;
import org.barracudamvc.examples.ex3.events.Level10Event;
import org.barracudamvc.examples.ex3.events.Level1Event;
import org.barracudamvc.examples.ex3.events.Level2Event;
import org.barracudamvc.examples.ex3.events.Level3Event;
import org.barracudamvc.examples.ex3.events.Level4Event;
import org.barracudamvc.examples.ex3.events.Level5Event;
import org.barracudamvc.examples.ex3.events.Level6Event;
import org.barracudamvc.examples.ex3.events.Level7Event;
import org.barracudamvc.examples.ex3.events.Level8Event;
import org.barracudamvc.examples.ex3.events.Level9Event;
import org.barracudamvc.examples.ex3.events.RenderTestScreenEvent;
//import org.barracudamvc.experimental.html.*;


/**
 * This mini-app is just used for stress testing, to see what 
 * kind of performance level degradation we experience as the 
 * event heirarchy depth increases.
 *
 * The way this works is simple. If you invoke a Level1Event,
 * the dispatcher will fire a RequestEvent, HttpRequestEvent,
 * and then the actual Level1Event. If you invoke a Level10Event
 * there will be 9 additional events generated because of the 
 * Polymorphic nature of the request hierarchy.
 */
public class TestScreen extends DefaultEventGateway {

    //public constants
    protected static final Logger logger = Logger.getLogger(TestScreen.class.getName());

    //this defines the various event handlers
    private ListenerFactory levelFactory =                 new DefaultListenerFactory() {public BaseEventListener getInstance() {return new LevelHandler();}                public String getListenerID() {return getID(LevelHandler.class);}};                
    private ListenerFactory renderFactory =             new DefaultListenerFactory() {public BaseEventListener getInstance() {return new RenderTestHandler();}            public String getListenerID() {return getID(RenderTestHandler.class);}};                

    //private vars

    /**
     * Public constructor
     */
    public TestScreen() {
        //specify who's interested in what
        specifyLocalEventInterests(levelFactory, Level1Event.class);
        specifyLocalEventInterests(levelFactory, Level2Event.class);
        specifyLocalEventInterests(levelFactory, Level3Event.class);
        specifyLocalEventInterests(levelFactory, Level4Event.class);
        specifyLocalEventInterests(levelFactory, Level5Event.class);
        specifyLocalEventInterests(levelFactory, Level6Event.class);
        specifyLocalEventInterests(levelFactory, Level7Event.class);
        specifyLocalEventInterests(levelFactory, Level8Event.class);
        specifyLocalEventInterests(levelFactory, Level9Event.class);
        specifyLocalEventInterests(levelFactory, Level10Event.class);
        specifyLocalEventInterests(renderFactory, RenderTestScreenEvent.class);
    }


    //------------------------------------------------------------
    //             Model 2 - Controller Event Handlers
    //------------------------------------------------------------
    /**
     * LevelHandlers - handle the request to get the test 
     * screen.
     */
    class LevelHandler extends DefaultBaseEventListener {
        public void handleControlEvent(ControlEventContext context) throws EventException, ServletException, IOException {
            //unpack the necessary entities from the context
            BaseEvent event = context.getEvent();
            DispatchQueue queue = context.getQueue();

            //in this case, we're not really doing anything special, so just
            //redirect to the RenderLogin view
            logger.debug("Handling event:"+event);
            BaseEvent newEvent = new RenderTestScreenEvent();
            newEvent.setSource(event);
            queue.addEvent(newEvent);
        }
    }
    

    //------------------------------------------------------------
    //                Model 2 - View Event Handlers
    //------------------------------------------------------------
    /**
     * RenderTestHandler - handle the request to render the test 
     * screen
     */
    class RenderTestHandler extends DefaultBaseEventListener {
        public void handleViewEvent(ViewEventContext context) throws EventException, ServletException, IOException {
            HttpServletResponse resp = context.getResponse();

            //csc_061202.1 - added    
            //set the caching hdrs (allow this page to be cached by browser)
            resp.setHeader("Cache-Control","max-age=0");
            resp.setDateHeader("Last-Modified", System.currentTimeMillis());

            //build the response
            resp.setContentType("text/html");
            PrintWriter out = resp.getWriter();
            
            //header            
            String title = "Simple Test Screen";
            out.println ("<html>");
            out.println ("  <head>");
            out.println ("    <title>"+title+"</title>");
            out.println ("  </head>");
            out.println ("  <body>");
            out.println ("    <font face=\"Arial\">");

            //body
            out.println ("      <p>"+title);
            
            //footer
            out.println ("    </font>");
            out.println ("  </body>");
            out.println ("</html>");
        }
    }
}

