package org.barracudamvc.examples.xmlc;

import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;
import org.barracudamvc.core.comp.AbstractBlockIterator;
import org.barracudamvc.core.comp.BlockIterator;
import org.barracudamvc.core.comp.IterativeModel;
import org.barracudamvc.core.event.ApplicationGateway;
import org.barracudamvc.core.event.BaseEventListener;
import org.barracudamvc.core.event.DefaultEventGateway;
import org.barracudamvc.core.event.DefaultListenerFactory;
import org.barracudamvc.core.event.ListenerFactory;
import org.barracudamvc.core.event.helper.BTemplateViewHandler;
import org.barracudamvc.core.event.helper.EventForwardingFactory;
import org.barracudamvc.core.util.dom.DOMWriter;
import org.barracudamvc.core.util.dom.DefaultDOMWriter;
import org.barracudamvc.examples.xmlc.events.CompTest10;
import org.barracudamvc.examples.xmlc.events.RenderCompTest10;
import org.barracudamvc.plankton.data.ObjectRepository;


/**
 * A sample screen (does the exact same thing as CompEx10, but routes to here
 * using an event, rather than a servlet url)
 */
public class CompEx10b extends DefaultEventGateway {

    //public constants
    protected static final Logger logger = Logger.getLogger(CompEx10b.class.getName());

    //set up our factories
    private ListenerFactory getTestFactory =         new EventForwardingFactory(new RenderCompTest10());
    private ListenerFactory renderTestFactory =     new DefaultListenerFactory() {public BaseEventListener getInstance() {return new RenderTestHandler();}            public String getListenerID() {return getID(RenderTestHandler.class);}};
    
    /**
     * Public constructor
     */
    public CompEx10b() {
        //specify who's interested in what
        specifyLocalEventInterests(getTestFactory, CompTest10.class);
        specifyLocalEventInterests(renderTestFactory, RenderCompTest10.class);
    }

    //------------------------------------------------------------
    //                Model 2 - View Event Handlers
    //------------------------------------------------------------
    /**
     * Handle a request to render the page
     */
    class RenderTestHandler extends BTemplateViewHandler {

        //add in all the common models, plus a local model
        public Object getTemplateModels() {
            return (new CompEx10()).getLocalModel();
        }

        //specify the template class
        public Class getTemplateClass() {
            ObjectRepository lor = ObjectRepository.getLocalRepository();
            HttpServletRequest req = (HttpServletRequest) lor.getState(ApplicationGateway.HTTP_SERVLET_REQUEST);
            boolean useBlockIterator = (req.getParameter("$usebi")!=null);    
            return (useBlockIterator ? CompEx10aHTML.class : CompEx10HTML.class);
        }

        //specify the dom writer
        public DOMWriter getDOMWriter() {
            return new DefaultDOMWriter();
        }

        //provide a block iterator - only necessary if part of the template needs to be streamed back directly
        public BlockIterator getIterator(String key) {
            return new AbstractBlockIterator() {
                IterativeModel lm = (new CompEx10()).getLocalModel();   //this block iterator will use one model repeatedly, and delegate to it as needed
            
                public boolean hasNext() {return lm.hasNext();}
            
                public boolean loadNext() {
                    return true;
                }
            
                public Object getTemplateModels() {return lm;}
            };
        }
    }
}

