/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: HttpConverter.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.plankton.http;

import gnu.regexp.RE;
import gnu.regexp.REException;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;

import org.apache.log4j.Logger;

/**
 * <p>This class provides a simple series of static methods
 * to convert a Map to a URL String and vica-versa</p>
 */
public class HttpConverter {

    //public constants
    protected static final Logger logger = Logger.getLogger(HttpConverter.class.getName());

//csc_070804_1_start
    public static RE ere1 = null;
    public static RE ere2 = null;
    public static RE dre1 = null;
    public static RE dre2 = null;
    public static final char ESC_CHAR = '\u0001';
    public static final char ESC_AMP = '\u00A5';

    //static initializer
    static {
        try {
            ere1 = new RE(""+ESC_CHAR);
            ere2 = new RE("&");
            dre1 = new RE(""+ESC_CHAR+ESC_AMP);
            dre2 = new RE(""+ESC_CHAR+ESC_CHAR);
        } catch (REException e) {
            logger.error("Unexpected REException: "+e+" creating REs ", e);
        }
    }
//csc_070804_1_end

    /**
     * Convert the values in a Map to a URL String (ie. key1=val1&key2=val2&...)
     */
    public static String cvtMapToURLString(Map map) {
        return cvtMapToURLString(map, "&");
    }

    /**
     * Convert the values in a Map to a URL String (ie. key1=val1&key2=val2&...). You
     * can specify an alternate delimeter here.
     */
    public static String cvtMapToURLString(Map map, String delimiter) {
        //eliminate the obvious
        if (map==null) return null;
        
        //if the map is not an instance of TreeMap, convert it so
        //that the keys are ordered by default. By doing this automatically,
        //we ensure that you can count on the order always being the same - 
        //you can easily tell if url parameters are the same between requests 
        //simply by comparing the resulting strings
        if (!(map instanceof SortedMap)) {    //csc_013104_1
            map = new TreeMap(map);
        }
    
        //run through the map and build a new string
        StringBuffer sb = new StringBuffer(200);
        String sep = "";
        Iterator it = map.keySet().iterator();
        while (it.hasNext()) {
            Object key = it.next();
            Object value = map.get(key);
            if (value instanceof Set) {
                Set set = (Set) value;
                Iterator it2 = set.iterator();
                while (it2.hasNext()) {
                    sb.append(concat(sep, key, it2.next()));
                }                            
            } else {
                sb.append(concat(sep, key, value));
            }
            sep = delimiter;
        }
        return sb.toString();
    }
    
    private static String concat(String sep, Object key, Object value) {
//csc_070804_1        return sep+key.toString()+"="+value.toString();
        return sep+encode(key.toString())+"="+encode(value.toString()); //csc_070804_1
    }
    
    /**
     * Convert the values in a URL String (ie. key1=val1&key2=val2&...) to a Map
     * of key/val pairs
     */
    public static Map cvtURLStringToMap(String paramStr) {
        return cvtURLStringToMap(paramStr, "&");
    }
    
    /**
     * Convert the values in a URL String (ie. key1=val1&key2=val2&...) to a Map
     * of key/val pairs. You can specify an alternate delimeter here.
     */
    public static Map cvtURLStringToMap(String paramStr, String delimiter) {
        //build the HashMap
//csc_010404_1        HashMap map = new HashMap(10);
        TreeMap map = new TreeMap();    //csc_010404_1
        int spos = 0;
        int epos = -1;
        int eqpos = -1;
        if (paramStr.startsWith("?")) paramStr = paramStr.substring(1); //csc_010404_1
        int max = paramStr.length();
        while (spos>=0 && spos<max) {
            eqpos = paramStr.indexOf("=",spos+1);
            if (eqpos<0) break;
            epos = paramStr.indexOf(delimiter,eqpos+1);
            if (epos<0 && eqpos>-1) epos = max;
            if (eqpos>spos || eqpos<epos) {
//csc_070804_1                String key = paramStr.substring (spos,eqpos);
//csc_070804_1                String value = paramStr.substring (eqpos+1, epos);
                String key = decode(paramStr.substring (spos,eqpos));       //csc_070804_1
                String value = decode(paramStr.substring (eqpos+1, epos));  //csc_070804_1
                if (map.containsKey(key)) {
                    Object mapVal = map.get(key);
                    if (mapVal instanceof Set) {
                        ((Set) mapVal).add(value);
                    } else {
                        Set set = new HashSet();
                        map.put(key, set);                    
                        set.add(mapVal);
                        set.add(value);
                    }
                } else {
                    map.put(key, value);
                }
            }
            spos = epos+1;
        }
        return map;
    }
    
//csc_070804_1_start
    /**
     * Encode a String - converts & to %& and % to %%
     */
    public static String encode(String s) {
        if (s==null) return null;
        s = ere1.substituteAll(s, ""+ESC_CHAR+ESC_CHAR);
        s = ere2.substituteAll(s, ""+ESC_CHAR+ESC_AMP);
        return s;
    }

    /**
     * Decode an encoded String - converts %& to & and %% to %
     */
    public static String decode(String s) {
        if (s==null) return null;
        s = dre1.substituteAll(s, "&");
        s = dre2.substituteAll(s, ""+ESC_CHAR);
        return s;
    }

    public static void main(String[] args) {
        try {
            String url = "hello 100% - Ma & Pa";
            System.out.println("orig:    "+url);
            url = encode(url);
            System.out.println("encoded: "+url);
            url = decode(url);
            System.out.println("decoded: "+url);
        } catch (Exception e) {
            System.out.println("unexpected exception: "+e);
            e.printStackTrace();
        }
    }
//csc_070804_1_end
}



