/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: LocaleTestSuiteSetup.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.testbed;

import java.util.Locale;

import junit.extensions.TestSetup;
import junit.framework.Test;

//
// a test setup that will set the default locale in setUp()
// and reset it in tearDown(). Used for running test cases in different locales
// (see TestDateValidator.java and TestDateRangeValidator.java)
//
public class LocaleTestSuiteSetup extends TestSetup {

    //===========================================================================
    // CONSTRUCTORS
    //===========================================================================
    public static String TEST_STRING = "foo99"; //please leave this var in here as the TestObjectRepositoryAssembler will use it to see if we can set a static var without instantiating the class

    /**
     * Default constructor
     *
     * @param theTest a Test object
     * @param theNewLocale the preferred local with which to test
     */
    public LocaleTestSuiteSetup(Test theTest, Locale theNewLocale) {
        super(theTest);
        this.myNewLocale = theNewLocale;
    }

    //===========================================================================
    // QUERY METHODS
    //===========================================================================

    //===========================================================================
    // MUTATOR METHODS
    //===========================================================================

    public void setUp() {
        this.myOriginalLocale = Locale.getDefault();
        Locale.setDefault(this.myNewLocale);
    }

    public void tearDown() {
        Locale.setDefault(this.myOriginalLocale);
    }

    //===========================================================================
    // HELPER METHODS
    //===========================================================================

    //===========================================================================
    // DATA MEMBERS
    //===========================================================================

    private Locale myOriginalLocale = null;
    private Locale myNewLocale = null;

    //===========================================================================
    // STATIC DATA MEMBERS
    //===========================================================================

}
