package org.barracudamvc.testbed.workbench.jdbc;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.io.ByteArrayOutputStream;
import java.io.PrintStream;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import org.barracudamvc.testbed.workbench.util.gui.Containers;
import org.barracudamvc.testbed.workbench.util.gui.Cursors;
import org.barracudamvc.testbed.workbench.util.gui.DefaultComponentListener;

/**
 * JDBC Tester
 */
public class JDBCTester extends JPanel {

    //public vars (these are set up to connect to a Microsoft SQL server db; you can modify
    //them accordingly. You will need to provide your own JDBC drivers in the classpath)
    public static String DEFAULT_URL = "jdbc:JSQLConnect://localhost/database=Master/user=sa";
    public static String DEFAULT_DRIVERS = "com.jnetdirect.jsql.JSQLDriver";
    public static String DEFAULT_USER = "";
    public static String DEFAULT_PWD = "";
    public static String DEFAULT_SQL_STR = "SELECT * FROM sysdatabases";
    
    //debug vars
    public static int showDebug = 0;

    //working vars
    private static String sep = System.getProperty("line.separator");
    private boolean isConnected = false;
    private Database db = null;
    private String jdbcServerURL = null;
    private String jdbcDriverName = null;
    private String dbUserName = null;
    private String dbPassword = null;

    //objects
    private boolean isInitialized = false;

    //buttons & controls
    //...Settings panel
    private JTextField jtfURL = null;
    private JTextField jtfDriver = null;
    private JTextField jtfUser = null;
    private JTextField jtfPwd = null;
    private JButton jbConnect = null;
    private JButton jbDisconnect = null;
    //...Main panel
    private JTabbedPane jtpTabs = null;
    //...Query panel
    private JPanel jpQuery = null;
    private JTextArea jtaQuery = null;
    //...Results panel
    private JPanel jpResults = null;
    private JTextArea jtaResults = null;
    //...Footer panel
    private JButton jbQuery = null;
    private JButton jbClear = null;

    
    /**
     * Public constructor
     */
    public JDBCTester() {
        //set up a listener so when this guy is exposed we hear about it
        this.addComponentListener(new LocalComponentListener());
    }
    
    //============================================================
    //        Model / Controller info for the Master View
    //============================================================
    /**
     * This method is responsible for laying out the
     * UI components. 
     *
     * @throws Exception
     */
    private void initViews() throws Exception {

        //general
        this.setLayout(new GridBagLayout());
        LocalActionAdapter actionAdapter = new LocalActionAdapter();
        LocalKeyListener keyListener = new LocalKeyListener();
        Insets buttonInsets = new Insets(0, 5, 0, 5);


        //settings panel
        JPanel jpSettings = new JPanel();
        jpSettings.setLayout(new GridBagLayout());
        this.add(jpSettings, new GridBagConstraints(0, GridBagConstraints.RELATIVE, 1, 1, 1.0, 0.0, 
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 0, 5), 0, 0));
        //...JDBC URL
        JLabel jlbURL = new JLabel("JDBC URL:");
        jpSettings.add(jlbURL, new GridBagConstraints(0, GridBagConstraints.RELATIVE, 1, 1, 0.0, 0.0, 
            GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(2, 5, 2, 5), 0, 0));
        jtfURL = new JTextField(DEFAULT_URL);
        jpSettings.add(jtfURL, new GridBagConstraints(1, GridBagConstraints.RELATIVE, 1, 1, 1.0, 0.0, 
            GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2, 5, 2, 5), 0, 0));
        //...JDBC Driver
        JLabel jlbDriver = new JLabel("JDBC Driver:");
        jpSettings.add(jlbDriver, new GridBagConstraints(0, GridBagConstraints.RELATIVE, 1, 1, 0.0, 0.0, 
            GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(2, 5, 2, 5), 0, 0));
        jtfDriver = new JTextField(DEFAULT_DRIVERS);
        jpSettings.add(jtfDriver, new GridBagConstraints(1, GridBagConstraints.RELATIVE, 1, 1, 1.0, 0.0, 
            GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(2, 5, 2, 5), 0, 0));
        //...User
        JLabel jlbUser = new JLabel("Username:");
        jpSettings.add(jlbUser, new GridBagConstraints(0, GridBagConstraints.RELATIVE, 1, 1, 0.0, 0.0, 
            GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(2, 5, 2, 5), 0, 0));
        JPanel jpUserPwd = new JPanel();
        jpUserPwd.setLayout(new GridBagLayout());
        jpSettings.add(jpUserPwd, new GridBagConstraints(1, GridBagConstraints.RELATIVE, 1, 1, 1.0, 0.0, 
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
        jtfUser = new JTextField(DEFAULT_USER,10);
        jpUserPwd.add(jtfUser, new GridBagConstraints(0, GridBagConstraints.RELATIVE, 1, 1, 0.0, 0.0, 
            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2, 5, 2, 5), 0, 0));
        //...Pwd
        JLabel jlbPwd = new JLabel("Password:");
        jpUserPwd.add(jlbPwd, new GridBagConstraints(1, GridBagConstraints.RELATIVE, 1, 1, 0.0, 0.0, 
            GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(2, 5, 2, 5), 0, 0));
        jtfPwd = new JTextField(DEFAULT_PWD,10);
        jpUserPwd.add(jtfPwd, new GridBagConstraints(2, GridBagConstraints.RELATIVE, 1, 1, 0.0, 0.0, 
            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(2, 5, 2, 5), 0, 0));
        //...Filler (to push the fields over)
        jpUserPwd.add(new JLabel(" "), new GridBagConstraints(3, GridBagConstraints.RELATIVE, 1, 1, 1.0, 0.0, 
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
        //...Connect
        jbConnect = new JButton("Connect");
        jbConnect.setToolTipText("Connect to the Database");
        jbConnect.setMargin(buttonInsets);
        jbConnect.addActionListener(actionAdapter);
        jpSettings.add(jbConnect, new GridBagConstraints(99, GridBagConstraints.RELATIVE, 1, 1, 0.0, 0.0, 
            GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
        //...Disconnect
        jbDisconnect = new JButton("Disconnect");
        jbDisconnect.setToolTipText("Disconnect from the Database");
        jbDisconnect.setMargin(buttonInsets);
        jbDisconnect.addActionListener(actionAdapter);
        jpSettings.add(jbDisconnect, new GridBagConstraints(99, GridBagConstraints.RELATIVE, 1, 1, 0.0, 0.0, 
            GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));



        //main panel
        jtpTabs = new JTabbedPane();
        jtpTabs.setTabPlacement(SwingConstants.BOTTOM);
        this.add(jtpTabs, new GridBagConstraints(0, GridBagConstraints.RELATIVE, 1, 1, 1.0, 1.0, 
            GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 0, 5), 0, 0));

        //...query panel
        jpQuery = new JPanel();
        jpQuery.setLayout(new GridBagLayout());
        jtpTabs.addTab("SQL", null, jpQuery, "Create a JDBC Query/Update");
        jtaQuery = new JTextArea(DEFAULT_SQL_STR);
        jtaQuery.addKeyListener(keyListener);
        JScrollPane jspQuery = new JScrollPane(jtaQuery);
        jpQuery.add(jspQuery, new GridBagConstraints(0, GridBagConstraints.RELATIVE, 1, 1, 1.0, 1.0, 
            GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));

        //...results panel
        jpResults = new JPanel();
        jpResults.setLayout(new GridBagLayout());
        jtpTabs.addTab("Results", null, jpResults, "View the results of a JDBC Query/Update");
        jtaResults = new JTextArea();
        jtaResults.addKeyListener(keyListener);
        JScrollPane jspResults = new JScrollPane(jtaResults);
        jpResults.add(jspResults, new GridBagConstraints(0, GridBagConstraints.RELATIVE, 1, 1, 1.0, 1.0, 
            GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 5, 5, 5), 0, 0));



        //Footer Panel
        JPanel jpFooter = new JPanel();
        jpFooter.setLayout(new GridBagLayout());
        this.add(jpFooter, new GridBagConstraints(0, GridBagConstraints.RELATIVE, 1, 1, 1.0, 0.0, 
            GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 5, 5, 5), 0, 0));
        //...filler
        jpFooter.add(new JLabel(" "), new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 1.0, 0.0, 
            GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
        //...Query
        jbQuery = new JButton("Execute SQL");
        jbQuery.setToolTipText("Execute the Query/Update SQL");
        jbQuery.setMargin(buttonInsets);
        jbQuery.addActionListener(actionAdapter);
        jpFooter.add(jbQuery, new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, 
            GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 5), 0, 0));
        //...Clear
        jbClear = new JButton("Clear");
        jbClear.setToolTipText("Clear the visible window");
        jbClear.setMargin(buttonInsets);
        jbClear.addActionListener(actionAdapter);
        jpFooter.add(jbClear, new GridBagConstraints(GridBagConstraints.RELATIVE, 0, 1, 1, 0.0, 0.0, 
            GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
    
        //setWorking = false
        setWorking(false);
    }

    /**
     * Connect to the database
     */    
    public void doConnect() {
        try {
            //set working = true
            setWorking(true);

            //get the setting
            jdbcServerURL = jtfURL.getText();
            jdbcDriverName = jtfDriver.getText();
            dbUserName = jtfUser.getText();
            dbPassword = jtfPwd.getText();
        
            //switch to the output window
            jtpTabs.setSelectedIndex(1);
        
            //connect to the database
            try {
                db = new Database(jdbcDriverName, jdbcServerURL, dbUserName, dbPassword);
                db.connectServer();
                jtaResults.append ("Connection negotiated"+sep);
                isConnected = true;
            } catch (Exception e) {
                db = null;
                jtaResults.append ("Connection failed:"+sep);
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                e.printStackTrace(new PrintStream(baos));
                jtaResults.append (baos.toString()+sep);
                java.awt.Toolkit.getDefaultToolkit().beep();
                JOptionPane.showMessageDialog(null, "Connection failed: "+e.toString(), "Error!", JOptionPane.ERROR_MESSAGE);
            }

        } finally {    
            //set working = false
            setWorking(false);
        }
    }
    
    /**
     * Disconnect from the database
     */    
    public void doDisconnect() {
        try {
            //set working = true
            setWorking(true);

            //switch to the output window
            jtpTabs.setSelectedIndex(1);
        
            //disconnect from the database
            try {
                db.disconnectServer();
                db = null;
                jtaResults.append ("Disconnected"+sep);
                isConnected = false;
            } catch (Exception e) {
                jtaResults.append ("Disconnection failed:"+sep);
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                e.printStackTrace(new PrintStream(baos));
                jtaResults.append (baos.toString()+sep);
                JOptionPane.showMessageDialog(null, "Disconnect failed: "+e.toString(), "Error!", JOptionPane.ERROR_MESSAGE);
            } 
        } finally {    
            //set working = false
            setWorking(false);
        }
    }
    
    /**
     * Execute a query / update
     */    
    public void doQuery() {
        try {
            //set working = true
            setWorking(true);

            //switch to the output window
            jtpTabs.setSelectedIndex(1);
        
            //prepare the sql String 
            String sql = ""+jtaQuery.getText().trim();
            //      (TODO: need to strip out separators)
            jtaResults.append("SQL:    "+sql+sep);
            jtaResults.append("==========================================================="+sep);
            if (sql.length()<1) {
                JOptionPane.showMessageDialog(null, "Invalid Query/Update String...Nothing was entered!", "Error!", JOptionPane.ERROR_MESSAGE);
                jtpTabs.setSelectedIndex(0);
                return;
            }
            
            //figure out whether we're querying or updating
            boolean isUpdate = true;
            if (sql.toLowerCase().startsWith("select")) isUpdate = false;

            //execute the query
            Statement stmt = null;
            try {            
                stmt = db.createStatement();
                if (isUpdate) {
                    //do the update
                    int rows = stmt.executeUpdate(sql);
                    db.commit();
                    jtaResults.append(rows+" rows updated"+sep+sep);
                    
                } else { 
                    //do the query
                    ResultSet rs = stmt.executeQuery(sql);

                    // get the metadata
                    int columns = rs.getMetaData().getColumnCount();
                    String columnName;
                    String columnType;
                    StringBuffer sb = new StringBuffer(500);
                    String delim = "";
                    for (int pos = 1; pos <= columns; pos++) {
                        columnName = rs.getMetaData().getColumnName(pos);
                        columnType = rs.getMetaData().getColumnTypeName(pos);
                        sb.append(delim+columnName+"["+columnType+"]");
                        if (pos==1) delim = ", ";
                    }
                    jtaResults.append("Cols:    "+sb.toString()+sep);
                    
                    // get the individual result set data
                    Object rowData;
                    int row = 0;
                    while (rs.next()) {
                        sb = new StringBuffer(500);
                        delim = "";
                        for (int pos = 1; pos <= columns; pos++) {
                            rowData = rs.getObject(pos);
                            sb.append(delim+rowData);
                            if (pos==1) delim = ", ";
                        }
                        jtaResults.append("Row "+(++row)+":    "+sb.toString()+sep);
                        jtaResults.append(" "+sep);
                    }
                    if (row==0) jtaResults.append("Rows:    (empty result set)"+sep+sep);
                }
                
            }
            catch (Exception e) { 
                jtaResults.append ("Query failed:"+sep);
                ByteArrayOutputStream baos = new ByteArrayOutputStream();
                e.printStackTrace(new PrintStream(baos));
                jtaResults.append (baos.toString()+sep);
                JOptionPane.showMessageDialog(null, "Query failed: "+e.getMessage(), "Error!", JOptionPane.ERROR_MESSAGE);
            } finally {
                try {stmt.close();} 
                catch (SQLException se) {}
            }
        } finally {    
            //set working = false
            setWorking(false);
        }
    }
    
    /**
     * Clear the current screen
     */    
    public void doClear() {
        try {
            //set working = true
            setWorking(true);

            //clear the currently exposed screen
            int curscreen = jtpTabs.getSelectedIndex();
            if (curscreen==0) jtaQuery.setText("");
            else if (curscreen==1) jtaResults.setText("");

        } finally {    
            //set working = false
            setWorking(false);
        }
    }
    
    private void setWorking(boolean val) {
        //set the cursors
        Cursors.setWorking(Containers.getRootContainer(this), val);

        //disable key fields
        jbConnect.setEnabled(!isConnected && !val);
        jbDisconnect.setEnabled(isConnected && !val);
        jbQuery.setEnabled(isConnected && !val);
        jbClear.setEnabled(!val);
    }    
    

    //-------------------- ComponentListener ---------------------
    /**
     * Handle component events
     */
    class LocalComponentListener extends DefaultComponentListener {
    
        /**
         * Invoked when the component has been made visible. 
         */
        public void componentShown(ComponentEvent e) {
            if (showDebug>1) System.out.println ("Shown:"+e);
            
            //init the screen and catch any unexpected errors
            if (!isInitialized) try {
                initViews();
                isInitialized = true;
            } catch (Exception be) {
                System.out.println ("Error initializing "+this);
                be.printStackTrace();
            }
        }
    }

    //-------------------- ActionListener ------------------------
    /**
     * Handle the actions for buttons
     */
    class LocalActionAdapter implements java.awt.event.ActionListener {

        /**
         * actionPerformed method
         */
        public void actionPerformed(ActionEvent e) {
            //handle the action
            Object o = e.getSource();
            if (o==jbConnect) doConnect();
            else if (o==jbDisconnect) doDisconnect();
            else if (o==jbQuery) doQuery();
            else if (o==jbClear) doClear();
            else {
                System.out.println ("Unhandled Action:"+o);            
            }
        }
    }
    
    //-------------------- KeyListener ---------------------------
    /**
     * Listener for keystrokes
     */
    class LocalKeyListener implements java.awt.event.KeyListener {

        public void keyPressed(KeyEvent e) {
            if (e.getKeyCode()==KeyEvent.VK_ENTER && e.isControlDown()) {
                jbQuery.doClick();
            }
        }

        public void keyReleased(KeyEvent e) {
            //ignore
        }
         
        public void keyTyped(KeyEvent e) {
            //ignore
        }
    }

    
}
