package org.barracudamvc.tutorials.comp;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.barracudamvc.core.comp.AbstractTemplateModel;
import org.barracudamvc.core.comp.BComponent;
import org.barracudamvc.core.comp.BLink;
import org.barracudamvc.core.comp.BTemplate;
import org.barracudamvc.core.comp.DefaultTemplateView;
import org.barracudamvc.core.comp.ViewContext;
import org.barracudamvc.core.comp.helper.ComponentGateway;
import org.barracudamvc.core.util.dom.DefaultDOMLoader;
import org.barracudamvc.tutorials.xmlc.HelloWorld1bHTML;
import org.barracudamvc.tutorials.xmlc.HelloWorld1b_footerHTML;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * Here's an example of HelloWorld using the BTemplate
 * component
 */
public class HelloWorld1b extends ComponentGateway {

    //xmlc factory (ok as static because its threadsafe)
//     private static XMLCFactory xmlcFactory = null;

    //-------------------- ComponentGateway ----------------------
    /**
     * Handle the default HttpRequest. 
     */
    public Document handleDefault (BComponent root, ViewContext vc, HttpServletRequest req, HttpServletResponse resp) throws IOException {
        //load the DOM object
//        if (xmlcFactory==null) xmlcFactory = new XMLCStdFactory(this.getClass().getClassLoader(), new StreamXMLCLogger());
//        XMLObject page = xmlcFactory.create(HelloWorld1bHTML.class);

        //load the localized DOM template
        Document page = DefaultDOMLoader.getGlobalInstance().getDOM(HelloWorld1bHTML.class, vc.getViewCapabilities().getClientLocale());
        Node node = page.getElementById("HelloWorld");

        //create the component and bind it to the view and the model
        BTemplate templateComp = new BTemplate(new HelloWorldModel());
        templateComp.setView(new DefaultTemplateView(node));

        //add the component to the root
        root.addChild(templateComp);
    
        //return the page
        return page;
    }

    //--------------- HelloWorldModel ------------------------
    /**
     * HelloWorldModel
     */
    class HelloWorldModel extends AbstractTemplateModel {
        //register the model by name
        public String getName() {
            return "HelloWorld";
        }
        
        //provide items by key
        public Object getItem(String key) {
            ViewContext vc = getViewContext();
            if (key.equals("Title")) return "Hello World 1 example";
            else if (key.equals("Header")) return new BLink("Email Christian", "mailto:christian.cryder@lutris.com");
            else if (key.equals("Descr")) return "This example illustrates how we can use models that return more than just Strings. Specifically, the model that backs the BTemplate component is returning a complex header and footer. For the header we are returning a BLink object that renders to the email address in the upper right hand corner. For the footer we are importing the HTML snippet that can be seen at the bottom of the page.";
            else if (key.equals("Hello")) return "Hello World! Hi Ma! Hi Pa!";
            else if (key.equals("Footer")) {
//                XMLObject footer = xmlcFactory.create(HelloWorld1b_footerHTML.class);
                Document footer = null;
                //load the localized DOM template
                try {footer = DefaultDOMLoader.getGlobalInstance().getDOM(HelloWorld1b_footerHTML.class, vc.getViewCapabilities().getClientLocale());}
                catch (IOException ioe) {logger.fatal ("Fatal Error loading DOM template:", ioe);}
                Node footerNode = ((HelloWorld1b_footerHTML)footer).getElementFooter();
//csc_072604_2                return vc.getElementFactory().getDocument().importNode(footerNode, true);
                return vc.getDocument().importNode(footerNode, true);   //csc_072604_2
            }
            else return super.getItem(key);
        }
    }
}
