/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: TestDefaultEventBroker.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.core.event;

import org.apache.log4j.*;

import org.barracudamvc.testbed.*;

//csc_121302.1 - created
/**
 * Unit tests for DefaultEventBroker (make sure we can register and look up ids/events
 * correctly
 */
public class TestDefaultEventBroker extends DefaultTestCase {
    //common vars (customize for every test class)
    private static String testClass = TestDefaultEventBroker.class.getName();
    private static Logger logger = Logger.getLogger("test."+testClass);

    //variables
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestDefaultEventBroker(String name) {
        super(name);
    }
    
    /**
     * Every test class should have a main method so it can be run 
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    
    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!
    
    /**
     * Simple test to verify that the component renders as expected
     */
    public void testListeners() throws InvalidClassException {
        if (logger.isInfoEnabled()) logger.info("testing event broker listeners");

        //create the event broker
        DefaultEventBroker eb = new DefaultEventBroker(null, ".event");
        String id = null;
        String result = null;
        
        //create some aliases 
        String classes[] = new String[] {
            "foo.blah.event.Test",
            "foo.blah.event.Blarney",
            "foo.blah.event2.Test"
        };
        for (int i=0; i<classes.length; i++) {
            eb.addAliases(classes[i], eb.getAliases(classes[i]), eb.eventXref);
        }       

        //now see what we have
/*        
        System.out.println ("");
        System.out.println ("Check @ point 1:");
        CollectionsUtil.printStackTrace(eb.eventXref, System.out);
*/        

        //now see if we get matches
        id = "Blarney";
        result = eb.matchEventClass(id);
        assertTrue("failed to find id:"+id+", got:"+result, classes[1].equals(result));
        id = "event2.Test";
        result = eb.matchEventClass(id);
        assertTrue("failed to find id:"+id+", got:"+result, classes[2].equals(result));
        id = "Test";
        try {
            result = eb.matchEventClass(id);
            fail("failed to throw exception on id:"+id+", got:"+result);
        } catch (InvalidClassException e) {
            //noop - this is what we expected
        }
        id = "BLARNEY";  //test upper case
        result = eb.matchEventClass(id);
        assertTrue("failed to find id:"+id+", got:"+result, classes[1].equals(result));
        id = "blarney";  //test lower case
        result = eb.matchEventClass(id);
        assertTrue("failed to find id:"+id+", got:"+result, classes[1].equals(result));
        id = "fOo.BlAh.EvEnT.bLaRnEy";  //test mixed case
        result = eb.matchEventClass(id);
        assertTrue("failed to find id:"+id+", got:"+result, classes[1].equals(result));
    }
}
