/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: TestScriptingType.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.core.view;

import junit.framework.*;

import org.apache.log4j.*;
import org.barracudamvc.testbed.*;


/**
 * This test verifies that the hierachical organization of 
 * ScriptingType classes. We do this by testing for what it should be 
 * an instanceof and what it shouldn't. The idea is that the ScriptingType
 * extension hierarchy is very important and we don't want it getting 
 * changed accidentally. This test should preclude that.
 */
public class TestScriptingType extends DefaultTestCase {
    //common vars (customize for every test class)
    private static String testClass = TestScriptingType.class.getName();
    private static Logger logger = Logger.getLogger("test."+testClass);

    //variables
    ScriptingType st = null;
    Class targetCl = null;
    
    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestScriptingType(String name) {
        super(name);
    }

    public TestScriptingType(String name, ScriptingType ist, Class itargetCl) {
        super(name);
        st = ist;
        targetCl = itargetCl;
    }
    
    /**
     * Every test class should have a main method so it can be run
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }

    /**
     * Return a suite of tests...
     */
	public static Test suite() {
        //create the test suite
		TestSuite suite = new TestSuite();
        
        //add the tests
        //...JAVASCRIPT_1x
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1x, ScriptingType.JavaScript1x.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1x, ScriptingType.JavaScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.JAVASCRIPT_1x, ScriptingType.WmlScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.JAVASCRIPT_1x, ScriptingType.None.class));
        //...JAVASCRIPT_1_0
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_0, ScriptingType.JavaScript10.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_0, ScriptingType.JavaScript1x.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_0, ScriptingType.JavaScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.JAVASCRIPT_1_0, ScriptingType.WmlScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.JAVASCRIPT_1_0, ScriptingType.None.class));
        //...JAVASCRIPT_1_1
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_1, ScriptingType.JavaScript11.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_1, ScriptingType.JavaScript10.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_1, ScriptingType.JavaScript1x.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_1, ScriptingType.JavaScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.JAVASCRIPT_1_1, ScriptingType.WmlScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.JAVASCRIPT_1_1, ScriptingType.None.class));
        //...JAVASCRIPT_1_2
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_2, ScriptingType.JavaScript12.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_2, ScriptingType.JavaScript11.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_2, ScriptingType.JavaScript10.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_2, ScriptingType.JavaScript1x.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_2, ScriptingType.JavaScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.JAVASCRIPT_1_2, ScriptingType.WmlScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.JAVASCRIPT_1_2, ScriptingType.None.class));
        //...JAVASCRIPT_1_3
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_3, ScriptingType.JavaScript13.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_3, ScriptingType.JavaScript12.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_3, ScriptingType.JavaScript11.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_3, ScriptingType.JavaScript10.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_3, ScriptingType.JavaScript1x.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.JAVASCRIPT_1_3, ScriptingType.JavaScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.JAVASCRIPT_1_3, ScriptingType.WmlScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.JAVASCRIPT_1_3, ScriptingType.None.class));
        //...WMLSCRIPT_1x
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1x, ScriptingType.WmlScript1x.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1x, ScriptingType.WmlScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.WMLSCRIPT_1x, ScriptingType.JavaScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.WMLSCRIPT_1x, ScriptingType.None.class));
        //...WMLSCRIPT_1_0
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_0, ScriptingType.WmlScript10.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_0, ScriptingType.WmlScript1x.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_0, ScriptingType.WmlScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.WMLSCRIPT_1_0, ScriptingType.JavaScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.WMLSCRIPT_1_0, ScriptingType.None.class));
        //...WMLSCRIPT_1_1
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_1, ScriptingType.WmlScript11.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_1, ScriptingType.WmlScript10.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_1, ScriptingType.WmlScript1x.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_1, ScriptingType.WmlScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.WMLSCRIPT_1_1, ScriptingType.JavaScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.WMLSCRIPT_1_1, ScriptingType.None.class));
        //...WMLSCRIPT_1_2
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_2, ScriptingType.WmlScript12.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_2, ScriptingType.WmlScript11.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_2, ScriptingType.WmlScript10.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_2, ScriptingType.WmlScript1x.class));
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.WMLSCRIPT_1_2, ScriptingType.WmlScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.WMLSCRIPT_1_2, ScriptingType.JavaScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.WMLSCRIPT_1_2, ScriptingType.None.class));
        //...NONE
        suite.addTest(new TestScriptingType("testInstance", ScriptingType.NONE, ScriptingType.None.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.NONE, ScriptingType.JavaScript.class));
        suite.addTest(new TestScriptingType("testNotInstance", ScriptingType.NONE, ScriptingType.WmlScript.class));

        //finally return the suite
        return suite;
    }


    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!

    /**
     * Make sure that the class being tested IS an instance of the 
     * target class
     */
    public void testInstance() {
        if (logger.isInfoEnabled()) logger.info("testing instanceof");
        assertTrue(st+" not an instanceof "+targetCl, targetCl.isAssignableFrom(st.getClass()));
    }

    /**
     * Make sure that the class being tested is NOT an instance of the 
     * target class
     */
    public void testNotInstance() {
        if (logger.isInfoEnabled()) logger.info("testing !instanceof");
        assertTrue(st+" is an instanceof "+targetCl+" (and it shouldn't be!)", !targetCl.isAssignableFrom(st.getClass()));
    }
}
